#!/bin/bash

#
# Copyright (c) 2010 Ingres Corp.
#

TOPDIR="`cat .topdir`"
IVWDIR="$TOPDIR/scripts/ingres_vectorwise"

usage()
{
cat << EOF
usage: $0 -d <db> [-s <n>] [-t <n>]

The script runs TPC-H ACID Consistency Tests

OPTIONS:
    -h              show this message
    -d  <database>  database name
    -s  <number>    number of streams (default=2)
    -t  <number>    number of transactions (default=100)
EOF
}

DATABASE=
STREAMS=2
NTRAN=100
while getopts "d:s:t:" opt; do
    case $opt in
        d)
            DATABASE=$OPTARG
            ;;
        s)
        	STREAMS=$OPTARG
        	;;
        t)
        	NTRAN=$OPTARG
        	;;
        ?)
            usage
            exit 1
            ;;
    esac
done

if [ -z "$DATABASE" ]; then
	usage
	exit 1
fi

if [ $STREAMS -lt 2 ]; then
	echo "Error: Number of streams must be >= 2"
	exit 1
fi

if [ $NTRAN -lt 100 ]; then
	echo "Error: Number of transactions must be >= 100"
	exit 1
fi

echo "======================="
echo "TPC-H Consistency Tests"
echo "======================="
echo "Test Started at: `date`"
echo

# truncate HISTORY table
#
echo "Truncate the HISTORY table:"
echo "delete from HISTORY;\p\g" | sql $DATABASE

# create directory to store test-related data
#
OUTPUT_DIR="$IVWDIR/acid/cons/output"
mkdir -p $OUTPUT_DIR

# generate NTRAN random numbers for each stream in STREAMS
#
for ((i=0;i<${STREAMS};i++)); do
	$IVWDIR/acid/acidkeys -b $DATABASE -c $NTRAN > $OUTPUT_DIR/stream${i}_rnd
	echo "random numbers for stream $i stored in $OUTPUT_DIR/stream${i}_rnd"
done

###################
# STEP 1
###################
echo
echo ">>> STEP 1"
echo

# Obtain 10 ORDERKEYs of each stream and check consistency condition 
# (since number of streams >= 2, at least 10 distinct values are verified)
#
for ((i=0;i<${STREAMS};i++)); do
	KEYS=`head -n 10 $OUTPUT_DIR/stream${i}_rnd | awk '{printf "%d ", $1}'`
		
	for j in $KEYS; do
		$IVWDIR/acid/cons/consist_condition -b $DATABASE -o ${j} >> $OUTPUT_DIR/step1_stream${i}_output		
	done
	echo "	output for stream ${i} in $OUTPUT_DIR/step1_stream${i}_output"
done

echo
echo "STEP 1 COMPLETED."
echo

###################
# STEP 2
###################
echo
echo ">>> STEP 2"
echo

# Submit ACID transactions (NTRAN per STREAM)
#
for ((i=0;i<${STREAMS};i++)); do
	echo "  Starting $NTRAN ACID transactions for Stream ${i} at `date`"
	cat $OUTPUT_DIR/stream${i}_rnd | $IVWDIR/acid/acidtrans -b $DATABASE > $OUTPUT_DIR/step2_stream${i}_output &
done

wait
echo "  Ending ACID transactions at `date`"
echo

# show where output is stored
#
for ((i=0;i<${STREAMS};i++)); do
	echo "  Check step 2 output for stream${i}: $OUTPUT_DIR/step2_stream${i}_output"
done

echo
echo "STEP 2 COMPLETED."
echo

###################
# STEP 3
###################
echo
echo ">>> STEP 3"
echo

# Using the KEYS collected in STEP 1, re-verify the consistency 
#
for ((i=0;i<${STREAMS};i++)); do
	KEYS=`head -n 10 $OUTPUT_DIR/stream${i}_rnd | awk '{printf "%d ", $1}'`
			
	for j in $KEYS; do
		$IVWDIR/acid/cons/consist_condition -b $DATABASE -o ${j} >> $OUTPUT_DIR/step3_stream${i}_output
	done
	echo "	output for stream ${i} in $OUTPUT_DIR/step3_stream${i}_output"
done

echo
echo "STEP 3 COMPLETED."
echo

# show HISTORY table
echo
echo "Contents of the HISTORY table:"
echo "select * from HISTORY;\p\g" | sql $DATABASE

echo
echo "Test Ended at: `date`"
