#!/bin/bash

echo Script Arguments: $0 $*

#
# Copyright (c) 2010 Ingres Corp.
#

TOPDIR="`cat .topdir`"        
IVWDIR="$TOPDIR/scripts/ingres_vectorwise"
source $TOPDIR/scripts/kit_profile || er "$0: could not source kit_profile"

unset DATABASE
unset SCALE_FACTOR
unset DBDATA
unset QGEN_DEFAULT_PARAMS
SET_NUM=0
REFRESH=1
OUTPUT_DIR="$IVWDIR/output"
SEED_FILE="$IVWDIR/seed_file"
STATS=0
STATS_FLAG="-nst"
RF_FLAG="-rf"
SCHEDRS_FLAG="-prf"
RUNS=2

usage()
{
cat << EOF
$1

usage: $0 flags

The performance test consists of two runs. A run consists of one execution of
the Power test followed by one execution of the Throughput test.

Required flags:
   -d     <name>    database name
   -df    <dir>     location of the Refresh Data Set files
   -s     <number>  scale factor
   -ns    <number>  number of query streams

Optional flags:
   -nprf            refresh stream starts after the completion of query streams
   -prf             refresh stream starts in parallel with the query streams (default)
   -l     <number>  When -prf is enabled, the delay in seconds before starting
	                the refresh stream (zero = no delay) (default value=$DELAY)
   -o     <dir>     where the tests artifacts will be stored (default=${OUTPUT_DIR})
   -sf    <file>    seed file name (default=${SEED_FILE})
   -rf              run refresh functions (default)
   -nrf             do not run refresh functions
   -st              collect system statistics
   -nst             do not collect system statistics (default)
   -h               this help page
       
EOF
exit 1
}

# process the command line parameters
while [ $# -ne 0 ]
do
    case $1 in
    -d)     shift;
            DATABASE=$1
        ;;
    -df)    shift;
            DBDATA=$1
        ;;
    -s)     shift;
            SCALE_FACTOR=$1
        ;;
    -o)     shift;
            OUTPUT_DIR=$1
        ;;
    -sf)    shift;
            SEED_FILE=$1
        ;;
    -st)    STATS=1
	    STATS_FLAG="-st"
        ;;
    -nst)   STATS=0
	    STATS_FLAG="-nst"
        ;;
    -rf)    REFRESH=1
        ;;
    -nrf)   REFRESH=0
	    RF_FLAG="-nrf"
        ;;
    -ns)    shift;
	    STREAMS=$1
	;;
    -prf)   SCHEDRS_FLAG="-prf"
        ;;
    -nprf)  SCHEDRS_FLAG="-nprf"
	;;
    *)		usage;
            ;;
    esac
    shift
done

if [ -z ${DATABASE} ]; then
	usage "ERROR: database name is required."
fi

if [ -z ${DBDATA} ]; then
	usage "ERROR: location of the Refresh Data Set files is required."
fi

if [ -z ${SCALE_FACTOR} ]; then
	usage "ERROR: scale factor is required."
fi
	
if [ -z ${STREAMS} ]; then
	usage "ERROR: number of streams is required."
fi

if [ ${REFRESH} -eq 1 ]; then
	# check if the file that stores the last RF data set number used exists
	if [ ! -f $IVWDIR/last_rfset_${DATABASE} ]; then
		er "$IVWDIR/last_rfset_${DATABASE} file does not exist. Test Aborted."
	fi

	# Get last RF data set number used
	LAST_SET_NUM=`cat $IVWDIR/last_rfset_${DATABASE}`

	SET_NUM=`expr ${LAST_SET_NUM} + 1`
fi

# Determine a unique identifer for this test run.
if test -f $IVWDIR/run_number; then
    read RUN_NUMBER < $IVWDIR/run_number
else
    RUN_NUMBER=1
fi
TAG=$RUN_NUMBER

RUN_NUMBER=`expr $RUN_NUMBER + 1`
echo $RUN_NUMBER > $IVWDIR/run_number

#appending TAG into OUTPUT_DIR
PERF_OUTPUT_DIR="$OUTPUT_DIR/$TAG"
mkdir -p $PERF_OUTPUT_DIR/results

if [ ! -f $SEED_FILE ]; then
    er "$SEED_FILE does not exist. Test Aborted."
fi
SEED=`cat $SEED_FILE`

echo "================================================="
echo "            TPC-H Performance Test"
echo "================================================="
echo " "


j=0
for i in `seq 1 $RUNS`; do
	echo " "
	echo "****************"
	echo "***  RUN $i  ***"
	echo "****************"

	$IVWDIR/power_test.sh -d ${DATABASE} -s ${SCALE_FACTOR} -df ${DBDATA} -o $OUTPUT_DIR ${RF_FLAG} ${STATS_FLAG} || \
	     er "RUN $i. Power Test Failed."
	export PT${i}=`expr $RUN_NUMBER + $j`
	j=`expr $j + 1`

	$IVWDIR/tput_test.sh -d ${DATABASE} -s ${SCALE_FACTOR} -df ${DBDATA} -o $OUTPUT_DIR -ns ${STREAMS} ${RF_FLAG} ${STATS_FLAG} ${SCHEDRS_FLAG} || \
	     er "RUN $i. Throughput Test Failed."
	export TT${i}=`expr $RUN_NUMBER + $j`
	j=`expr $j + 1`
done

echo " "
echo '=================================='
echo ' Performance Test completed.'
echo '=================================='

echo "Metrics:" | tee -a $PERF_OUTPUT_DIR/performance_test_report.txt
for i in `seq 1 $RUNS`; do
	eval tmp1=\$PT$i
	eval tmp2=\$TT$i

	power=`cat $OUTPUT_DIR/$tmp1/results/power_metric`
	tput=`cat $OUTPUT_DIR/$tmp2/results/throughput_metric`

	# compute performance metric, round to the nearest tenth
	perf=`echo ${power} ${tput} | awk '{x=sqrt($1*$2); printf("%.1f", int(x*10+0.5)/10);}'`

	echo "RUN #$i. Power Metric: $power" | tee -a $PERF_OUTPUT_DIR/performance_test_report.txt
	echo "RUN #$i. Throughput Metric: $tput" | tee -a $PERF_OUTPUT_DIR/performance_test_report.txt
	echo "RUN #$i. Performance Metric: $perf" | tee -a $PERF_OUTPUT_DIR/performance_test_report.txt
	echo " " | tee -a $PERF_OUTPUT_DIR/performance_test_report.txt
done

echo " " | tee -a $PERF_OUTPUT_DIR/performance_test_report.txt
echo "Reports:" | tee -a $PERF_OUTPUT_DIR/performance_test_report.txt

for i in `seq 1 $RUNS`; do
        eval tmp1=\$PT$i
        eval tmp2=\$TT$i
        echo "RUN #$i. Power Test: $OUTPUT_DIR/$tmp1/power_test_report.txt" | 
		tee -a $PERF_OUTPUT_DIR/performance_test_report.txt
        echo "RUN #$i. Throughput Test: $OUTPUT_DIR/$tmp2/tput_test_report.txt" |
		tee -a $PERF_OUTPUT_DIR/performance_test_report.txt
done

echo
echo "Performance Test Report: see $PERF_OUTPUT_DIR/performance_test_report.txt"
