#!/bin/bash

#
# Copyright (c) 2010 Ingres Corp.
#

TOPDIR="`cat .topdir`"        
IVWDIR="$TOPDIR/scripts/ingres_vectorwise"
source $TOPDIR/scripts/kit_profile || er "$0: could not source kit_profile"

unset DATABASE
unset SCALE_FACTOR
unset DBDATA
unset QGEN_DEFAULT_PARAMS
SET_NUM=0
REFRESH=1
OUTPUT_DIR="$IVWDIR/output"
SEED_FILE="$IVWDIR/seed_file"
STATS=0
VALIDATION=0
TEST="Power Test"
unset PAR

# Power Test Stream
STREAM=0

usage()
{
cat << EOF
$1

usage: $0 flags

Required flags:
   -d     <name>    database name
   -df    <dir>     location of the Refresh Data Set files
   -s     <number>  scale factor

Optional flags:            
   -o     <dir>     where the tests artifacts will be stored (default=${OUTPUT_DIR})
   -sf    <file>    seed file name (default=${SEED_FILE})
   -rf              run refresh functions (default)
   -nrf             do not run refresh functions
   -us    <number>  update set number for refresh functions
	                default is to use 'last_rfset_<db name> used + 1'
   -st              collect system statistics
   -nst             do not collect system statistics (default)
   -val             Execute a Validation test
                      * QGEN uses default substitution parameters (seed file is discarded)
                      * Do not run Refresh Functions
   -h               this help page

EOF
exit 1
}

# process the command line parameters
while [ $# -ne 0 ]
do
    case $1 in
    -d)     shift;
            DATABASE=$1
        ;;
    -df)    shift;
            DBDATA=$1
        ;;
    -s)     shift;
            SCALE_FACTOR=$1
        ;;
    -o)     shift;
            OUTPUT_DIR=$1
        ;;
    -sf)    shift;
            SEED_FILE=$1
        ;;
    -st)    STATS=1
        ;;
    -nst)   STATS=0
        ;;
    -rf)    REFRESH=1
        ;;
    -nrf)   REFRESH=0
        ;;
    -us)    shift;
            SET_NUM=$1
        ;;
    -val)   VALIDATION=1
            REFRESH=0
		;;
    *)      usage;
            ;;
    esac
    shift
done

if [ -z ${DATABASE} ]; then	
	usage "ERROR: database name is required."
fi

if [ $REFRESH -eq 1 ]; then
	if [ -z ${DBDATA} ]; then
		usage "ERROR: location of the Refresh Data Set files is required."	
	fi

	if [ ! -d ${DBDATA} ]; then
		er "directory of the Refresh Data Set files (${DBDATA}) does not exist."	
	fi

	#compute what parallelism level we should use
	del_file=$DBDATA/delete.u1.
	for ((f=1; f<1000; f++));do
		if [ ! -r ${del_file}$f ];then
			break;
		fi
	done
	if [ $f -eq 1 ];then
		er "update directory $DBDATA does not contain update files"
	fi
	PAR="-par `expr $f - 1`"
fi

if [ -z ${SCALE_FACTOR} ]; then
	usage "ERROR: scale factor is required."
fi

if [ ${REFRESH} -eq 1 -a ${SET_NUM} -eq 0 ]; then
	# check if the file that stores the last RF data set number used exists
	if [ ! -f $IVWDIR/last_rfset_${DATABASE} ]; then
		er "$IVWDIR/last_rfset_${DATABASE} file does not exist. Test Aborted."
	fi

	# Get last RF data set number used
	LAST_SET_NUM=`cat $IVWDIR/last_rfset_${DATABASE}`

	SET_NUM=`expr ${LAST_SET_NUM} + 1`
fi

# Determine a unique identifer for this test run.
if test -f $IVWDIR/run_number; then
    read RUN_NUMBER < $IVWDIR/run_number
else
    RUN_NUMBER=1
fi
TAG=$RUN_NUMBER

RUN_NUMBER=`expr $RUN_NUMBER + 1`
echo $RUN_NUMBER > $IVWDIR/run_number

#appending TAG into OUTPUT_DIR
OUTPUT_DIR="$OUTPUT_DIR/$TAG"
mkdir -p $OUTPUT_DIR/results

echo "Creating ${OUTPUT_DIR}/queries subdirectory"
# location to store query executable text
QUERIESDIR=$OUTPUT_DIR/queries
mkdir -p $QUERIESDIR

if [ ${VALIDATION} -eq 1 ]; then
    REFRESH=0
    QGEN_DEFAULT_PARAMS="-d"
    TEST="Validation Test"
    echo "Test Type: Power Test - VALIDATION TEST" >> $OUTPUT_DIR/power_test_report.txt
fi

if [ ! -f $SEED_FILE ]; then
    er "$SEED_FILE does not exist. Test Aborted."
fi

SEED=`cat $SEED_FILE`

if [ ${STATS} -eq 1 ]; then
    # Start collecting system statistics.
    $TOPDIR/scripts/start_sysstats.sh -o $OUTPUT_DIR || er "could not start sysstats"
    trap 'echo "*** Caught SIGTERM/SIGINT, stopping sysstats" && $TOPDIR/scripts/stop_sysstats.sh && exit 1' TERM INT QUIT ABRT
fi

# Generate queries text files
$IVWDIR/gen_query.sh ${QGEN_DEFAULT_PARAMS} -e ${SEED} -s ${SCALE_FACTOR} -m ${STREAM} -q ${QUERIESDIR}

echo "================================================="
echo "  TPC-H ${TEST}"
echo "================================================="

# Power Test start time
s_time=`$GTIME`
human_s_time=`echo $s_time | $IVWDIR/unix2human.awk`
echo "Start time: $human_s_time"
echo

# RF1
#
if [ ${REFRESH} -eq 1 ]; then
    $IVWDIR/run_rf1.sh -d ${DATABASE} -df ${DBDATA} -o ${OUTPUT_DIR} \
      -s ${SET_NUM} ${PAR} || er "RF1 failed."
fi

# Execute query stream
#
$IVWDIR/run_query_stream.sh -d ${DATABASE} -s ${STREAM} -o ${OUTPUT_DIR} \
  -q ${QUERIESDIR} || er "query stream failed."

# RF2
#
if [ ${REFRESH} -eq 1 ]; then
    $IVWDIR/run_rf2.sh -d ${DATABASE} -df ${DBDATA} -o ${OUTPUT_DIR} \
      -s ${SET_NUM} ${PAR} || er "RF2 failed."
      
    # store last RF set number used
	echo ${SET_NUM} > $IVWDIR/last_rfset_${DATABASE}
fi

echo '======================================='
echo "  ${TEST} Completed"
echo '======================================='

# Power Test end time
e_time=`$GTIME`
human_e_time=`echo $e_time | $IVWDIR/unix2human.awk`
echo "End time: $human_e_time"

# calculate and report power test time
echo "Start time of Power Test: $human_s_time" >> $OUTPUT_DIR/power_test_report.txt
echo "End time of Power Test: $human_e_time" >> $OUTPUT_DIR/power_test_report.txt

diff_time=`calc $e_time-$s_time`
echo "Power Test Duration: $diff_time seconds" >> $OUTPUT_DIR/power_test_report.txt

if [ ${VALIDATION} -eq 1 ]; then
	echo "Seed: Validation Test - Using validation subsitution parameters" >> $OUTPUT_DIR/power_test_report.txt
else	
	echo "Seed: $SEED" >> $OUTPUT_DIR/power_test_report.txt
fi
echo "Stream: $STREAM" >> $OUTPUT_DIR/power_test_report.txt

if [ ${STATS} -eq 1 ]; then
    # Stop collecting system statistics.
    echo "Stop collecting system statistics."
    $TOPDIR/scripts/stop_sysstats.sh || er "$0: could not run stop_sysstats.sh"
fi

report_stream_times ${OUTPUT_DIR}/results/stream${STREAM}/timing_intervals >> $OUTPUT_DIR/power_test_report.txt

# Sort query times by query # and calculate query duration
sort -n -k1 ${OUTPUT_DIR}/results/stream${STREAM}/timing_intervals | awk \
   -f $IVWDIR/query_stream_results.awk -v sf=${SCALE_FACTOR} >> $OUTPUT_DIR/power_test_report.txt

if [ ${REFRESH} -eq 1 ]; then
	cat $OUTPUT_DIR/results/rf1/times${SET_NUM} >> ${OUTPUT_DIR}/results/stream${STREAM}/timing_intervals
	cat $OUTPUT_DIR/results/rf2/times${SET_NUM} >> ${OUTPUT_DIR}/results/stream${STREAM}/timing_intervals
	
	report_rf ${OUTPUT_DIR} power_test_report.txt
fi

# look for errors
ERROR=`cat $OUTPUT_DIR/power_test_report.txt | grep -i error | wc -l`

# calculate power metric
POWER_METRIC=`cat ${OUTPUT_DIR}/results/stream${STREAM}/timing_intervals | awk -f $IVWDIR/power_metric.awk -v sf=${SCALE_FACTOR}` 

if [ $ERROR -gt 0 ]; then
	POWER_METRIC=`echo "$POWER_METRIC ($ERROR ERROR)"`
fi	

# report power metric
echo $POWER_METRIC > ${OUTPUT_DIR}/results/power_metric
echo "TPC-H Power@${SCALE_FACTOR}: $POWER_METRIC" | tee -a $OUTPUT_DIR/power_test_report.txt

echo
echo "report: see $OUTPUT_DIR/power_test_report.txt"

exit $ERROR
