#!/bin/bash

#
# Copyright (c) 2010 Ingres Corp.
#

TOPDIR="`cat .topdir`"       
IVWDIR="$TOPDIR/scripts/ingres_vectorwise"
source $TOPDIR/scripts/kit_profile || er "$0: could not source kit_profile"

error()
{
	touch ${OUTPUT_DIR}/ERROR
	er "$1"
}

unset DATABASE
unset SCALE_FACTOR
unset DBDATA
unset STREAMS
OUTPUT_DIR="$IVWDIR/output"
SEED_FILE="$IVWDIR/seed_file"
STATS=0
REFRESH=1
PARALLEL_RF=1
DELAY=0
unset PAR

usage()
{
cat << EOF
$1

usage: $0 flags

Required flags:
   -d     <name>    database name
   -df    <dir>     location of the Refresh Data Set files
   -s     <number>  scale factor
   -ns    <number>  number of query streams

Optional flags:
   -nprf            refresh stream starts after the completion of query streams
   -prf             refresh stream starts in parallel with the query streams (default)
   -l     <number>  When -prf is enabled, the delay in seconds before starting
	                the refresh stream (zero = no delay) (default value=$DELAY)
   -o     <dir>     where the tests artifacts will be stored (default=${OUTPUT_DIR})
   -sf    <file>    seed file name (default=${SEED_FILE})
   -rf              run refresh functions (default)
   -nrf             do not run refresh functions
   -st              collect system statistics
   -nst             do not collect system statistics (default)
   -h               this help page
        
EOF
exit 1
}

# process the command line parameters
while [ $# -ne 0 ]
do
    case $1 in
    -d)     shift;
            DATABASE=$1
        ;;
    -df)    shift;
            DBDATA=$1
        ;;
    -ns)    shift;
	    STREAMS=$1
	;;
    -s)     shift;
            SCALE_FACTOR=$1
        ;;
    -o)     shift;
            OUTPUT_DIR=$1
        ;;
    -sf)    shift;
            SEED_FILE=$1
        ;;
    -st)    STATS=1
        ;;
    -nst)   STATS=0
        ;;
    -rf)    REFRESH=1
        ;;
    -nrf)   REFRESH=0
        ;;
    -prf)   PARALLEL_RF=1
	;;
    -nprf)  PARALLEL_RF=0
	;;
    -l)     shift;
	    DELAY=$1
	;;
    *)      usage;
            ;;
    esac
    shift
done

if [ -z ${DATABASE} ]; then
	usage "ERROR: database name is required."
fi

if [ -z ${DBDATA} ]; then
	usage "ERROR: location of the Refresh Data Set files is required."
fi

if [ ! -d ${DBDATA} ]; then
	er "directory of the Refresh Data Set files (${DBDATA}) does not exist."	
fi

if [ -z ${SCALE_FACTOR} ]; then
	usage "ERROR: scale factor is required."
fi

if [ -z ${STREAMS} ]; then
	usage "ERROR: Number of streams is required."
fi

if [ ${STREAMS} -lt 2 ]; then
	er "Number of streams must be greater than 1."
fi

#compute what parallelism level we should use
del_file=$DBDATA/delete.u1.
for ((f=1; f<1000; f++));do
	if [ ! -r ${del_file}$f ];then
		break;
	fi
done
if [ $f -eq 1 ];then
	er "update directory $DBDATA does not contain update files"
fi
PAR="-par `expr $f - 1`"

# seed file exists?
if [ ! -f $SEED_FILE ]; then
    er "$SEED_FILE does not exist. Test Aborted."
fi

# check if the file that stores the last RF data set number used exists
if [ ! -f $IVWDIR/last_rfset_${DATABASE} ]; then
    er "$IVWDIR/last_rfset_${DATABASE} file does not exist. Test Aborted."
fi

# Determine a unique identifer for this test run.
#
if test -f $IVWDIR/run_number; then
    read RUN_NUMBER < $IVWDIR/run_number
else
    RUN_NUMBER=1
fi
TAG=$RUN_NUMBER

RUN_NUMBER=`expr $RUN_NUMBER + 1`
echo $RUN_NUMBER > $IVWDIR/run_number

#appending TAG into OUTPUT_DIR
OUTPUT_DIR="$OUTPUT_DIR/$TAG"
mkdir -p $OUTPUT_DIR/results

echo "Creating ${OUTPUT_DIR}/queries subdirectory"
# location to store queries executable text
QUERIESDIR=$OUTPUT_DIR/queries
mkdir -p $QUERIESDIR

SEED=`cat $SEED_FILE`

# Geenerate queries text
for ((i=1;i<=${STREAMS};i++)); do
	SEED=`expr ${SEED} + 1`
	echo "Stream ${i} - Seed: $SEED" >> $OUTPUT_DIR/tput_test_report.txt
	$IVWDIR/gen_query.sh -e ${SEED} -s ${SCALE_FACTOR} -m ${i} \
         -q ${QUERIESDIR} || er "could not generate query text"
done

# Calculate Start and End RF data set to be used
#

# Get last RF data set number used
LAST_SET_NUM=`cat $IVWDIR/last_rfset_${DATABASE}`

START_SET_NUM=`expr ${LAST_SET_NUM} + 1`
END_SET_NUM=`expr ${START_SET_NUM} + ${STREAMS} - 1`

# Start collecting system statistics
#
if [ ${STATS} -eq 1 ]; then
    $TOPDIR/scripts/start_sysstats.sh -o $OUTPUT_DIR || er "could not start sysstats"
    trap 'echo "*** Caught SIGTERM/SIGINT, stopping sysstats" && $TOPDIR/scripts/stop_sysstats.sh && exit 1' TERM INT QUIT ABRT
fi

echo "================================================="
echo "  TPC-H Throughput Test"
echo "================================================="

# Tput Test start time
s_time=`$GTIME`
human_s_time=`echo $s_time | $IVWDIR/unix2human.awk`
echo "Start time: $human_s_time"
echo

# Start query streams
for ((i=1;i<=${STREAMS};i++)); do
	$IVWDIR/run_query_stream.sh -d ${DATABASE} -s ${i} -o ${OUTPUT_DIR} \
	   -q ${QUERIESDIR} || error "query stream ${i} failed." &
	export QSPID${i}=$!
done

if [ ${PARALLEL_RF} -eq 1 ]; then
	sleep ${DELAY}
else
	# wait for the query streams to complete before starting the refresh stream
	for ((i=1;i<=${STREAMS};i++)); do
	        eval LPID=\$QSPID$i
        	wait $LPID
	done
fi

if [ ${REFRESH} -eq 1 ]; then
	$IVWDIR/run_rf_stream.sh -d ${DATABASE} -df ${DBDATA} -o ${OUTPUT_DIR} -ss ${START_SET_NUM} \
           -es ${END_SET_NUM} ${PAR} || error "refresh stream failed." &
	RSPID=$!
    
    # Wait for the refresh stream to complete
    wait $RSPID
fi

if [ ${PARALLEL_RF} -eq 1 ]; then
	# wait for the query streams to complete
	for ((i=1;i<=${STREAMS};i++)); do
		eval LPID=\$QSPID$i
        wait $LPID
    done
fi

# Tput Test end time
e_time=`$GTIME`
human_e_time=`echo $e_time | $IVWDIR/unix2human.awk`
echo "End time: $human_e_time"

if [ -f ${OUTPUT_DIR}/ERROR ]; then
        echo '======================================='
        echo "   Throughput Test FAILED!"
        echo '======================================='
else
	echo '======================================='
	echo "   Throughput Test Completed"
	echo '======================================='
fi

if [ ${STATS} -eq 1 ]; then
    # Stop collecting system statistics.
    echo "Stop collecting system statistics."
    $TOPDIR/scripts/stop_sysstats.sh || er "$0: could not run stop_sysstats.sh"
fi

# calculate measurement interval
MI=`calc $e_time - $s_time`
MI=`echo $MI | awk '{printf("%.1f",($1*10+0.5)/10);}'`

echo >> $OUTPUT_DIR/tput_test_report.txt
echo "Scale Factor: ${SCALE_FACTOR}" >> $OUTPUT_DIR/tput_test_report.txt
echo "Number of Streams: ${STREAMS}" >> $OUTPUT_DIR/tput_test_report.txt
echo " " >> $OUTPUT_DIR/tput_test_report.txt
echo "Measurement Interval" >> $OUTPUT_DIR/tput_test_report.txt
echo "====================" >> $OUTPUT_DIR/tput_test_report.txt
echo "Start time of Throughput Test: $human_s_time" >> $OUTPUT_DIR/tput_test_report.txt
echo "End time of Throughput Test: $human_e_time" >> $OUTPUT_DIR/tput_test_report.txt
echo "MI (seconds): $MI"  >> $OUTPUT_DIR/tput_test_report.txt
echo " " >> $OUTPUT_DIR/tput_test_report.txt

for ((i=1;i<=${STREAMS};i++)); do
	echo " " >> $OUTPUT_DIR/tput_test_report.txt
	echo "STREAM: ${i}" >> $OUTPUT_DIR/tput_test_report.txt
    report_stream_times ${OUTPUT_DIR}/results/stream${i}/timing_intervals >> $OUTPUT_DIR/tput_test_report.txt

	# Sort query times by query # and calculate query duration
	sort -n -k1 ${OUTPUT_DIR}/results/stream${i}/timing_intervals | awk \
	   -f $IVWDIR/query_stream_results.awk -v sf=${SCALE_FACTOR} >> $OUTPUT_DIR/tput_test_report.txt
done

# RF data
if [ ${REFRESH} -eq 1 ]; then
	report_rf ${OUTPUT_DIR} tput_test_report.txt
fi	

# look for errors
ERROR=`cat $OUTPUT_DIR/tput_test_report.txt | grep -i error | wc -l`

# compute Throughput metric, rounded to the nearest 0.1
TPUT_METRIC=`echo ${STREAMS} ${MI} ${SCALE_FACTOR} | awk '{x=(($1*22*3600)/$2)*$3; printf("%.1f", int(x*10+0.5)/10);}'`

if [ $ERROR -gt 0 ]; then
	TPUT_METRIC=`echo "$TPUT_METRIC ($ERROR ERRORS)"`
fi	

# report tput metric
echo $TPUT_METRIC > $OUTPUT_DIR/results/throughput_metric
echo " " >> $OUTPUT_DIR/tput_test_report.txt
echo "TPC-H Throughput@${SCALE_FACTOR}: $TPUT_METRIC" | tee -a $OUTPUT_DIR/tput_test_report.txt

echo
echo "report: see $OUTPUT_DIR/tput_test_report.txt"

exit $ERROR
